/* global BdfApi,Fql */

/**
 * Objet permettant de stocker des éléments récupérés par l'API Ajax * 
 * 
 * @constructor
  */
 BdfApi.Wrapper = function () {
    this.thesaurusMap = new Map();
    this.fichesMap = new Map();
};


/**
 * Retourne le thésaurus correspondant à la clé en argument
 * 
 * @param {string} key Clé du thésaurus 
 * @returns {BdfApi.Wrapper.Thesaurus} Objet Thesaurus
 */
BdfApi.Wrapper.prototype.getThesaurus = function (key) {
    return this.thesaurusMap.get(key);
};


/**
 * Retourne l'objet Fiches correspondant à la clé en argument
 * 
 * @param {string} key Clé d'un ensemble fiches
 * @returns {BdfApi.Wrapper.Fiches} Objet Fiches
 */
BdfApi.Wrapper.prototype.getFiches = function (key) {
    return this.fichesMap.get(key);
};


/**
 * Retourne une fonction d'appel de l'API, cette fonction prend comme argument
 * une fonction de retour pour permettre le chainage 
 * 
 * @param {BdfApi.Config} apiConfig Configuration à utiliser
 * @param {string} thesaurusName Nom du thésaurus
 * @param {Object} options Options de l'appel
 * @param {string} options.thesaurusKey Clé du thésaurus à utiliser au lieu du nom du thésaurus
 * @param {string} options.propertiesMapping Nom de l'export tabulaire à utiliser pour définir les propriétés d'un mot-clé
 * @returns {function} Fonction d'appel de l'API
 */
BdfApi.Wrapper.prototype.initThesaurus = function (apiConfig, thesaurusName, options) {
    var thesaurusMap = this.thesaurusMap;
    var thesaurusKey = thesaurusName;
    if ((options) && (options.thesaurusKey)) {
        thesaurusKey = options.thesaurusKey;
    }
    var requestParameters = {};
    if (options) {
        if (options.thesaurusKey) {
            thesaurusKey = options.thesaurusKey;
        }
        if (options.propertiesMapping) {
            requestParameters.tableexport = options.propertiesMapping;
        }
    }
    return function (callback) {
        BdfApi.Ajax.loadThesaurus(apiConfig, thesaurusName, {
            requestParameters: requestParameters,
            callback: function (data) {
                let wrapper = new BdfApi.Wrapper.Thesaurus(data);
                thesaurusMap.set(thesaurusKey, wrapper);
                if (callback) {
                    callback();
                }
            }
        });
    };
};


/**
 * Retourne une fonction d'appel de l'API, cette fonction prend comme argument
 * une fonction de retour pour permettre le chainage 
 * 
 * @param {BdfApi.Config} apiConfig Configuration à utiliser
 * @param {string} fichesKey Clé à utiliser pour conserver le résultat
 * @param {Object} options Options de l'appel
 * @param {Object} options.propertiesMapping Nom de l'export tabulaire à utiliser pour définir les propriétés d'un mot-clé
 * @param {Fql.FicheQuery} options.Query Instance de Fql.FicheQuery indiquant les fiches à récupérer
 * @param {Object} options.ql Objet correspondant à un objet Options utilisé en argument du constructeur de Fql.FicheQuery 
 * @param {Fql.FicheQuery} options.uniqueCorpus Indique que la recherche ne concerne qu'un seul corpus
 * @returns {function} Fonction d'appel de l'API
 */
BdfApi.Wrapper.prototype.initFiches = function (apiConfig, fichesKey, options) {
    var fichesMap = this.fichesMap;
    var uniqueCorpus = false;
    var requestParameters = {};
    var ficheQuery = null;
    if (options) {
        if (options.propertiesMapping) {
            requestParameters.tableexport = options.propertiesMapping;
        }
        if (options.ficheQuery) {
            ficheQuery = options.ficheQuery;
        } else if (options.ficheQL) {
            ficheQuery = options.ficheQL;
        } else if (options.ql) {
            ficheQuery = new Fql.FicheQuery(options.ql);
        }
        if (options.uniqueCorpus) {
            uniqueCorpus = true;
        }
    }
    if (!ficheQuery) {
        uniqueCorpus = true;
        ficheQuery = new Fql.FicheQuery({
            corpus: fichesKey
        });
    }
    return function (callback) {
        BdfApi.Ajax.loadFicheArray(apiConfig, ficheQuery, {
            requestParameters: requestParameters,
            callback: function (data) {
                let wrapper = new BdfApi.Wrapper.Fiches(data, uniqueCorpus);
                fichesMap.set(fichesKey, wrapper);
                if (callback) {
                    callback();
                }
            }
        });
    };
};

BdfApi.Wrapper.Thesaurus = function (thesaurusApiData) {
    var idMap = new Map();
    var idalphaMap = new Map();
    this.apiData = thesaurusApiData;
    this.idMap = idMap;
    this.idalphaMap = idalphaMap;
    _populate(thesaurusApiData.thesaurus.firstLevelArray);


    function _populate(array) {
        for(let motcle of array) {
            idMap.set(motcle.id, motcle);
            if (motcle.idalpha) {
                idalphaMap.set(motcle.idalpha, motcle);
            }
            _populate(motcle.childArray);
        }
    }
    
};

BdfApi.Wrapper.Thesaurus.prototype.getName = function () {
    return this.apiData.thesaurus.name;
};

BdfApi.Wrapper.Thesaurus.prototype.getFirstLevelArray = function () {
    return this.apiData.thesaurus.firstLevelArray;
};

BdfApi.Wrapper.Thesaurus.prototype.getMotcleById = function (id) {
	return this.idMap.get(id);
};

BdfApi.Wrapper.Thesaurus.prototype.getMotcleByIdalpha = function (idalpha) {
    return this.idalphaMap.get(idalpha);
};

BdfApi.Wrapper.Thesaurus.prototype.complete = function (completeFunction) {
    for(let motcle of this.idMap.values()) {
        completeFunction(motcle);
    }
};

BdfApi.Wrapper.Thesaurus.prototype.toMotcleArray = function () {
    var array = new Array();
    for(let motcle of this.idMap.values()) {
        array.push(motcle);
    }
    return array;
};

BdfApi.Wrapper.Fiches = function (ficheArrayApiData, uniqueCorpus) {
    this.apiData = ficheArrayApiData;
    this.uniqueCorpus = uniqueCorpus;
    var codeMap = new Map();
    var idMap = new Map();
    this.codeMap = codeMap;
    this.idMap = idMap;
    for(let ficheApiData of ficheArrayApiData.array) {
        codeMap.set(ficheApiData.code, ficheApiData);
        if (uniqueCorpus) {
            idMap.set(ficheApiData.id, ficheApiData);
        }
    }
};

BdfApi.Wrapper.Fiches.prototype.getFiche = function (corpus, id) {
    return this.codeMap.get(corpus + "/" + id);
};

BdfApi.Wrapper.Fiches.prototype.getFicheById = function (id) {
    return this.idMap.get(id);
};

BdfApi.Wrapper.Fiches.prototype.getFicheArray = function () {
    return this.apiData.array;
};

BdfApi.Wrapper.Fiches.prototype.complete = function (completeFunction) {
    for(let fiche of this.idMap.values()) {
        completeFunction(fiche);
    }
};
